RACE = {
    1: 'Human',
    2: 'Orc',
    3: 'Dwarf',
    4: 'Night Elf',
    5: 'Undead',
    6: 'Tauren',
    7: 'Gnome',
    8: 'Troll',
    9: 'Goblin',
    10: 'Blood Elf',
    11: 'Draenei',
    22: 'Worgen',
}

CLASS = {
    1: 'Warrior',
    2: 'Paladin',
    3: 'Hunter',
    4: 'Rogue',
    5: 'Priest',
    7: 'Shaman',
    8: 'Mage',
    9: 'Warlock',
    11: 'Druid',
    6: 'Death Knight',
}

QUALITY = {
    1: 'Common',
    2: 'Uncommon',
    3: 'Rare',
    4: 'Epic',
    5: 'Legendary',
}

RACE_TO_FACTION = {
    1: 'Alliance',
    2: 'Horde',
    3: 'Alliance',
    4: 'Alliance',
    5: 'Horde',
    6: 'Horde',
    7: 'Alliance',
    8: 'Horde',
    9: 'Horde',
    10: 'Horde',
    11: 'Alliance',
    22: 'Alliance',
}
